==> fancy_bottom_navigation-0.1.0.tar.gz: lib/fancy_bottom_navigation.dart <==
library fancy_bottom_navigation;

import 'package:flutter/material.dart';
import 'package:fancy_bottom_navigation/internal/tab_item.dart';
import 'package:fancy_bottom_navigation/paint/half_clipper.dart';
import 'package:fancy_bottom_navigation/paint/half_painter.dart';

const double CIRCLE_SIZE = 60;
const double ARC_HEIGHT = 70;
const double ARC_WIDTH = 90;
const double CIRCLE_OUTLINE = 10;
const double SHADOW_ALLOWANCE = 20;
const double BAR_HEIGHT = 60;

class FancyBottomNavigation extends StatefulWidget {
  FancyBottomNavigation(
      {@required this.tabs,
      @required this.context,
      @required this.onTabChangedListener,
      this.initialSelection = 0,
      this.circleColor,
      this.activeIconColor,
      this.inactiveIconColor,
      this.textColor,
      this.barBackgroundColor})
      : assert(onTabChangedListener != null),
        assert(context != null),
        assert(tabs != null),
        assert(tabs.length > 1 && tabs.length < 5);

  final Function(int position) onTabChangedListener;
  final Color circleColor;
  final Color activeIconColor;
  final Color inactiveIconColor;
  final Color textColor;
  final Color barBackgroundColor;
  final List<TabData> tabs;
  final int initialSelection;
  final BuildContext context;

  @override
  _FancyBottomNavigationState createState() => _FancyBottomNavigationState();
}

class _FancyBottomNavigationState extends State<FancyBottomNavigation>
    with TickerProviderStateMixin {
  IconData nextIcon = Icons.search;
  IconData activeIcon = Icons.search;

  int currentSelected = 0;
  double _circleAlignX = 0;
  double _circleIconAlpha = 1;

  Color circleColor;
  Color activeIconColor;
  Color inactiveIconColor;
  Color barBackgroundColor;
  Color textColor;

  OverlayEntry _overlayEntry;
  OverlayState _overlayState;

  @override
  void initState() {
    super.initState();
    activeIcon = widget.tabs[widget.initialSelection].iconData;
    _setSelected(widget.tabs[widget.initialSelection].key);
    circleColor = (widget.circleColor == null)
        ? (Theme.of(widget.context).brightness == Brightness.dark)
            ? Colors.white
            : Theme.of(widget.context).primaryColor
        : widget.circleColor;

    activeIconColor = (widget.activeIconColor == null)
        ? (Theme.of(widget.context).brightness == Brightness.dark)
            ? Colors.black54
            : Colors.white
        : widget.activeIconColor;

    barBackgroundColor = (widget.barBackgroundColor == null)
        ? (Theme.of(widget.context).brightness == Brightness.dark)
            ? Color(0xFF212121)
            : Colors.white
        : widget.barBackgroundColor;
    textColor = (widget.textColor == null)
        ? (Theme.of(widget.context).brightness == Brightness.dark)
            ? Colors.white
            : Colors.black54
        : widget.textColor;
    inactiveIconColor = (widget.inactiveIconColor == null)
        ? (Theme.of(widget.context).brightness == Brightness.dark)
            ? Colors.white
            : Theme.of(widget.context).primaryColor
        : widget.inactiveIconColor;

    WidgetsBinding.instance.addPostFrameCallback((_) {
      this._overlayState = Overlay.of(context);
      this._overlayEntry = this._createOverlayEntry();
      _overlayState.insert(this._overlayEntry);
    });

    if (_overlayEntry != null) {
      _overlayEntry.markNeedsBuild();
      _overlayState.setState(() {});
      this._overlayEntry = this._createOverlayEntry();
    }
  }

  OverlayEntry _createOverlayEntry() {
    debugPrint("Create Overlay Entry");
    return OverlayEntry(
        builder: (context) => IgnorePointer(
              child: AnimatedAlign(
                  duration: Duration(milliseconds: ANIM_DURATION),
                  curve: Curves.easeOut,
                  alignment: Alignment(_circleAlignX, 1),
                  child: Padding(
                    padding: const EdgeInsets.only(bottom: 15),
                    child: FractionallySizedBox(
                      widthFactor: 1 / widget.tabs.length,
                      child: Stack(
                        alignment: Alignment.center,
                        children: <Widget>[
                          SizedBox(
                            height:
                                CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                            width:
                                CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                            child: ClipRect(
                                clipper: HalfClipper(),
                                child: Container(
                                  child: Center(
                                    child: Container(
                                        width: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                        height: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                        decoration: BoxDecoration(
                                            color: Colors.white,
                                            shape: BoxShape.circle,
                                            boxShadow: [
                                              BoxShadow(
                                                  color: Colors.black12,
                                                  blurRadius: 8)
                                            ])),
                                  ),
                                )),
                          ),
                          SizedBox(
                              height: ARC_HEIGHT,
                              width: ARC_WIDTH,
                              child: CustomPaint(
                                painter: HalfPainter(barBackgroundColor),
                              )),
                          SizedBox(
                            height: CIRCLE_SIZE,
                            width: CIRCLE_SIZE,
                            child: Container(
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle, color: circleColor),
                              child: Padding(
                                padding: const EdgeInsets.all(0.0),
                                child: AnimatedOpacity(
                                  duration:
                                      Duration(milliseconds: ANIM_DURATION ~/ 5),
                                  opacity: _circleIconAlpha,
                                  child: Icon(
                                    activeIcon,
                                    color: activeIconColor,
                                  ),
                                ),
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            );
  }

  _setSelected(UniqueKey key) {
    int selected = widget.tabs.indexWhere((tabData) => tabData.key == key);

    setState(() {
      currentSelected = selected;
      _circleAlignX = -1 + (2 / (widget.tabs.length - 1) * selected);
      nextIcon = widget.tabs[selected].iconData;
    });

    if (_overlayState != null) {
      _overlayState.setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: Alignment.topCenter,
      children: <Widget>[
        Container(
          height: BAR_HEIGHT,
          decoration: BoxDecoration(color: barBackgroundColor, boxShadow: [
            BoxShadow(
                color: Colors.black12, offset: Offset(0, -1), blurRadius: 8)
          ]),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: widget.tabs
                .map((t) => TabItem(
                    uniqueKey: t.key,
                    selected: t.key == widget.tabs[currentSelected].key,
                    iconData: t.iconData,
                    title: t.title,
                    iconColor: inactiveIconColor,
                    textColor: textColor,
                    callbackFunction: (uniqueKey) {
                      int selected = widget.tabs
                          .indexWhere((tabData) => tabData.key == uniqueKey);
                      widget.onTabChangedListener(selected);
                      _setSelected(uniqueKey);
                      _initAnimationAndStart(_circleAlignX, 1);
                    }))
                .toList(),
          ),
        ),
      ],
    );
  }

  _initAnimationAndStart(double from, double to) {
    _circleIconAlpha = 0;

    Future.delayed(Duration(milliseconds: ANIM_DURATION ~/ 5), () {
      _overlayState.setState(() {});
      setState(() {
        activeIcon = nextIcon;
      });
    }).then((_) {
      Future.delayed(Duration(milliseconds: (ANIM_DURATION ~/ 5 * 3)), () {
        _overlayState.setState(() {});
        setState(() {
          _circleIconAlpha = 1;
        });
      });
    });
  }
}

class TabData {
  TabData({@required this.iconData, @required this.title});

  IconData iconData;
  String title;
  final UniqueKey key = UniqueKey();
}

==> fancy_bottom_navigation-0.1.1.tar.gz: lib/fancy_bottom_navigation.dart <==
library fancy_bottom_navigation;

import 'package:flutter/material.dart';
import 'package:fancy_bottom_navigation/internal/tab_item.dart';
import 'package:fancy_bottom_navigation/paint/half_clipper.dart';
import 'package:fancy_bottom_navigation/paint/half_painter.dart';

const double CIRCLE_SIZE = 60;
const double ARC_HEIGHT = 70;
const double ARC_WIDTH = 90;
const double CIRCLE_OUTLINE = 10;
const double SHADOW_ALLOWANCE = 20;
const double BAR_HEIGHT = 60;

class FancyBottomNavigation extends StatefulWidget {
  FancyBottomNavigation(
      {@required this.tabs,
      @required this.onTabChangedListener,
      this.initialSelection = 0,
      this.circleColor,
      this.activeIconColor,
      this.inactiveIconColor,
      this.textColor,
      this.barBackgroundColor})
      : assert(onTabChangedListener != null),
        assert(tabs != null),
        assert(tabs.length > 1 && tabs.length < 5);

  final Function(int position) onTabChangedListener;
  final Color circleColor;
  final Color activeIconColor;
  final Color inactiveIconColor;
  final Color textColor;
  final Color barBackgroundColor;
  final List<TabData> tabs;
  final int initialSelection;

  @override
  _FancyBottomNavigationState createState() => _FancyBottomNavigationState();
}

class _FancyBottomNavigationState extends State<FancyBottomNavigation>
    with TickerProviderStateMixin {
  IconData nextIcon = Icons.search;
  IconData activeIcon = Icons.search;

  int currentSelected = 0;
  double _circleAlignX = 0;
  double _circleIconAlpha = 1;

  Color circleColor;
  Color activeIconColor;
  Color inactiveIconColor;
  Color barBackgroundColor;
  Color textColor;

  OverlayEntry _overlayEntry;
  OverlayState _overlayState;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    activeIcon = widget.tabs[widget.initialSelection].iconData;
    _setSelected(widget.tabs[widget.initialSelection].key);
    circleColor = (widget.circleColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
        ? Colors.white
        : Theme.of(context).primaryColor
        : widget.circleColor;

    activeIconColor = (widget.activeIconColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
        ? Colors.black54
        : Colors.white
        : widget.activeIconColor;

    barBackgroundColor = (widget.barBackgroundColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
        ? Color(0xFF212121)
        : Colors.white
        : widget.barBackgroundColor;
    textColor = (widget.textColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
        ? Colors.white
        : Colors.black54
        : widget.textColor;
    inactiveIconColor = (widget.inactiveIconColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
        ? Colors.white
        : Theme.of(context).primaryColor
        : widget.inactiveIconColor;
  }

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      this._overlayState = Overlay.of(context);
      this._overlayEntry = this._createOverlayEntry();
      _overlayState.insert(this._overlayEntry);
    });

    if (_overlayEntry != null) {
      _overlayEntry.markNeedsBuild();
      _overlayState.setState(() {});
      this._overlayEntry = this._createOverlayEntry();
    }
  }

  OverlayEntry _createOverlayEntry() {
    debugPrint("Create Overlay Entry");
    return OverlayEntry(
        builder: (context) => IgnorePointer(
              child: AnimatedAlign(
                  duration: Duration(milliseconds: ANIM_DURATION),
                  curve: Curves.easeOut,
                  alignment: Alignment(_circleAlignX, 1),
                  child: Padding(
                    padding: const EdgeInsets.only(bottom: 15),
                    child: FractionallySizedBox(
                      widthFactor: 1 / widget.tabs.length,
                      child: Stack(
                        alignment: Alignment.center,
                        children: <Widget>[
                          SizedBox(
                            height:
                                CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                            width:
                                CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                            child: ClipRect(
                                clipper: HalfClipper(),
                                child: Container(
                                  child: Center(
                                    child: Container(
                                        width: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                        height: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                        decoration: BoxDecoration(
                                            color: Colors.white,
                                            shape: BoxShape.circle,
                                            boxShadow: [
                                              BoxShadow(
                                                  color: Colors.black12,
                                                  blurRadius: 8)
                                            ])),
                                  ),
                                )),
                          ),
                          SizedBox(
                              height: ARC_HEIGHT,
                              width: ARC_WIDTH,
                              child: CustomPaint(
                                painter: HalfPainter(barBackgroundColor),
                              )),
                          SizedBox(
                            height: CIRCLE_SIZE,
                            width: CIRCLE_SIZE,
                            child: Container(
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle, color: circleColor),
                              child: Padding(
                                padding: const EdgeInsets.all(0.0),
                                child: AnimatedOpacity(
                                  duration:
                                      Duration(milliseconds: ANIM_DURATION ~/ 5),
                                  opacity: _circleIconAlpha,
                                  child: Icon(
                                    activeIcon,
                                    color: activeIconColor,
                                  ),
                                ),
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            );
  }

  _setSelected(UniqueKey key) {
    int selected = widget.tabs.indexWhere((tabData) => tabData.key == key);

    setState(() {
      currentSelected = selected;
      _circleAlignX = -1 + (2 / (widget.tabs.length - 1) * selected);
      nextIcon = widget.tabs[selected].iconData;
    });

    if (_overlayState != null) {
      _overlayState.setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: Alignment.topCenter,
      children: <Widget>[
        Container(
          height: BAR_HEIGHT,
          decoration: BoxDecoration(color: barBackgroundColor, boxShadow: [
            BoxShadow(
                color: Colors.black12, offset: Offset(0, -1), blurRadius: 8)
          ]),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: widget.tabs
                .map((t) => TabItem(
                    uniqueKey: t.key,
                    selected: t.key == widget.tabs[currentSelected].key,
                    iconData: t.iconData,
                    title: t.title,
                    iconColor: inactiveIconColor,
                    textColor: textColor,
                    callbackFunction: (uniqueKey) {
                      int selected = widget.tabs
                          .indexWhere((tabData) => tabData.key == uniqueKey);
                      widget.onTabChangedListener(selected);
                      _setSelected(uniqueKey);
                      _initAnimationAndStart(_circleAlignX, 1);
                    }))
                .toList(),
          ),
        ),
      ],
    );
  }

  _initAnimationAndStart(double from, double to) {
    _circleIconAlpha = 0;

    Future.delayed(Duration(milliseconds: ANIM_DURATION ~/ 5), () {
      _overlayState.setState(() {});
      setState(() {
        activeIcon = nextIcon;
      });
    }).then((_) {
      Future.delayed(Duration(milliseconds: (ANIM_DURATION ~/ 5 * 3)), () {
        _overlayState.setState(() {});
        setState(() {
          _circleIconAlpha = 1;
        });
      });
    });
  }
}

class TabData {
  TabData({@required this.iconData, @required this.title});

  IconData iconData;
  String title;
  final UniqueKey key = UniqueKey();
}
